<?php

namespace FiloBlu\ScratchAndQuiz\Block\Adminhtml\ScratchAndQuizDetails\Edit\Tab;

class ScratchAndQuizDetails extends \Magento\Backend\Block\Widget\Form\Generic implements \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * @var \Magento\Store\Model\System\Store
     */
    protected $_systemStore;


    /**
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Data\FormFactory $formFactory
     * @param \Magento\Cms\Model\Wysiwyg\Config $wysiwygConfig
     * @param \Magento\Store\Model\System\Store $systemStore
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \Magento\Store\Model\System\Store $systemStore,
        \Magento\Cms\Model\Wysiwyg\Config $wysiwyg,

        array $data = []
    )
    {
        $this->_systemStore = $systemStore;
        $this->_wysiwyg = $wysiwyg;

        parent::__construct($context, $registry, $formFactory, $data);
    }

    /**
     * Init form
     *
     * @return void
     */
    protected function _construct()
    {
        parent::_construct();
        $this->setId('scratchandquizdetails_form');
        $this->setTitle(__('ScratchAndQuiz Details Information'));
    }

    /**
     * Prepare form
     *
     * @return $this
     */
    protected function _prepareForm()
    {
        $storeId = $this->getRequest()->getParam('store',0);

        $model = $this->_coreRegistry->registry('filoblu_scratchandquizdetails');

        $modelTranslations = $this->_coreRegistry->registry('filoblu_scratchandquizdetails_translations');

        $form = $this->_formFactory->create();

        $form->setHtmlIdPrefix('filoblu_scratchandquizdetails_');

        if ($storeId == 0) {

            $fieldset = $form->addFieldset(
                'base_fieldset',
                ['legend' => __('General Information'), 'class' => 'fieldset-wide']
            );

            if ($model->getId()) {
                $fieldset->addField('id', 'hidden', ['name' => 'id']);
            }

            if (!$model->getId()) {
                $model->setData('enabled', '1');
                $model->setData('cursor_size', '30');
                $model->setData('percentage', '50');
            }

            $fieldset->addField(
                'enabled',
                'select',
                [
                    'label' => __('Status'),
                    'title' => __('Status'),
                    'name' => 'enabled',
                    'required' => true,
                    'options' => ['1' => __('Enabled'), '0' => __('Disabled')]
                ]
            );

            $fieldset->addField(
                'cursor_size',
                'text',
                [
                    'name' => 'cursor_size',
                    'label' => __('Size of the eraser'),
                    'title' => __('Cursor Size'),
                    'required' => true,
                    'note' => 'Default value: 30'
                ]
            );
    
            $fieldset->addField(
                'percentage',
                'text',
                [
                    'name' => 'percentage',
                    'label' => __('Scratch Percentage %'),
                    'title' => __('Percentage'),
                    'required' => true,
                    'note' => 'Default value: 50'
                ]
            );
        }

        $fieldset = $form->addFieldset(
            'content_fieldset',
            ['legend' => __('Content Information'), 'class' => 'fieldset-wide']
        );

        $fieldset->addType('image', '\FiloBlu\ScratchAndQuiz\Block\Adminhtml\ScratchAndQuiz\Helper\Image');
        $fieldset->addType('file', '\FiloBlu\ScratchAndQuiz\Block\Adminhtml\ScratchAndQuiz\Helper\File');

        $fieldset->addField(
            'short_display_text',
            'editor',
            [
                'name' => 'short_display_text',
                'label' => __('Short Display Text'),
                'title' => __('Short Display Text'),
                'config' => $this->_wysiwyg->getConfig(),
                'wysiwyg' => true
            ]
        );

        $fieldset->addField(
            'long_display_text',
            'editor',
            [
                'name' => 'long_display_text',
                'label' => __('Long Display Text'),
                'title' => __('Long Display Text'),
                'config' => $this->_wysiwyg->getConfig(),
                'wysiwyg' => true
            ]
        );

        $fieldset->addField(
            'out_display_text',
            'editor',
            [
                'name' => 'out_display_text',
                'label' => __('Out Display Text'),
                'title' => __('Out Display Text'),
                'config' => $this->_wysiwyg->getConfig(),
                'wysiwyg' => true
            ]
        );

        $fieldset->addField(
            'overlay_image',
            'image',
            array(
                'name' => 'overlay_image',
                'label' => __('Overlay Image'),
                'title' => __('Overlay Image')
            )
        );


        $fieldset->addField(
            'bg_image',
            'image',
            array(
                'name' => 'bg_image',
                'label' => __('BG Image'),
                'title' => __('BG Image')
            )
        );

        $fieldset->addField(
            'scratched_image',
            'image',
            array(
                'name' => 'scratched_image',
                'label' => __('Scratched Image'),
                'title' => __('Scratched Image')
            )
        );

        if ($storeId == 0) {

            $fieldset->addField(
                'promocode',
                'text',
                [
                    'name' => 'promocode',
                    'label' => __('Promocode'),
                    'title' => __('Promocode'),
                    'required' => false
                ]
            );
        }

        $fieldset->addField(
            'cta_link',
            'text',
            [
                'name' => 'cta_link',
                'label' => __('CTA Link'),
                'title' => __('CTA Link'),
                'required' => false,
                'note' => 'You can use Magento variables. ex: {{store url=""}}'
            ]
        );


        if ($storeId != 0 && $modelTranslations->getId()) {
            $unserializedValues = unserialize($modelTranslations->getSerializedValues());
            $form->setValues($unserializedValues);
        }
        else {
            $form->setValues($model->getData());
        }
//        $form->setUseContainer(true);
        $this->setForm($form);

        return parent::_prepareForm();
    }

    /**
     * Prepare label for tab
     *
     * @return string
     */
    public function getTabLabel()
    {
        return __('ScratchAndQuiz Details');
    }

    /**
     * Prepare title for tab
     *
     * @return string
     */
    public function getTabTitle()
    {
        return $this->getTabLabel();
    }

    /**
     * Can show tab in tabs
     *
     * @return boolean
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * Tab is hidden
     *
     * @return boolean
     */
    public function isHidden()
    {
        return false;
    }
}
