<?php

namespace FiloBlu\ScratchAndQuiz\Controller\Adminhtml\ScratchAndQuiz;

use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;

/**
 * Cms page grid inline edit controller
 *
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class InlineEdit extends \Magento\Backend\App\Action
{
    /** @var JsonFactory */
    protected $jsonFactory;

    protected $scratchAndQuiz;

    /**
     * @param Context $context
     * @param JsonFactory $jsonFactory
     */
    public function __construct(
        Context $context,
        JsonFactory $jsonFactory,
        \FiloBlu\ScratchAndQuiz\Model\ScratchAndQuizFactory $scratchAndQuiz
    )
    {
        parent::__construct($context);
        $this->scratchAndQuiz = $scratchAndQuiz;
        $this->jsonFactory = $jsonFactory;
    }

    /**
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Json $resultJson */
        $resultJson = $this->jsonFactory->create();
        $error = false;
        $messages = [];

        $postItems = $this->getRequest()->getParam('items', []);

        if (!($this->getRequest()->getParam('isAjax') && count($postItems))) {
            return $resultJson->setData([
                'messages' => [__('Please correct the data sent.')],
                'error' => true,
            ]);
        }

        foreach (array_keys($postItems) as $scratchandquizId) {

            $model = $this->scratchAndQuiz->create();
            $model->load($scratchandquizId);
            $model->setData($postItems[$scratchandquizId]);

            $this->_eventManager->dispatch(
                'filoblu_scratchandquiz_prepare_save',
                ['scratchandquiz' => $model, 'request' => $this->getRequest()]
            );

            try {
                $model->save();
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $messages[] = $e->getMessage();
                $error = true;
            } catch (\RuntimeException $e) {
                $messages[] = $e->getMessage();
                $error = true;
            } catch (\Exception $e) {
                $messages[] = $e->getMessage();
                $error = true;
            }
        }

        return $resultJson->setData([
            'messages' => $messages,
            'error' => $error
        ]);
    }
}
