<?php
namespace FiloBlu\ScratchAndQuiz\Model\ResourceModel;

class ScratchAndQuiz extends \Magento\Framework\Model\ResourceModel\Db\AbstractDb
{
    /**
     * Event Manager
     *
     * @var \Magento\Framework\Event\ManagerInterface
     */
    protected $eventManager;

    protected $scratchandquizRelationTable;

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init('filoblu_scratchandquiz', 'id');
    }

    public function __construct(
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Framework\Model\ResourceModel\Db\Context $context
    )
    {
        $this->eventManager = $eventManager;

        parent::__construct($context);

        $this->scratchandquizRelationTable = $this->getTable('filoblu_scratchandquiz_relations');
    }

    protected function _afterSave(\Magento\Framework\Model\AbstractModel $object) {
        $this->saveDetailsRelation($object);

        return parent::_afterSave($object);
    }

    protected function saveDetailsRelation(\FiloBlu\ScratchAndQuiz\Model\ScratchAndQuiz $scratchAndQuiz) {
        $scratchAndQuiz->setIsChangedDetailsList(false);
        $id = $scratchAndQuiz->getId();

        $details = $scratchAndQuiz->getDetailsData();
        if ($details === null) {
            return $this;
        }
        $oldDetails = $scratchAndQuiz->getDetailsPosition();
        $insert = array_diff_key($details, $oldDetails);
        $delete = array_diff_key($oldDetails, $details);
        $update = array_intersect_key($details, $oldDetails);
        $_update = array();
        foreach ($update as $key => $settings) {
            if (isset($oldDetails[$key]) && $oldDetails[$key] != $settings['position']) {
                $_update[$key] = $settings;
            }
        }
        $update = $_update;
        $adapter = $this->getConnection();
        if (!empty($delete)) {
            $condition = ['filoblu_scratchandquiz_details_id IN(?)' => array_keys($delete), 'filoblu_scratchandquiz_id=?' => $id];
            $adapter->delete($this->scratchandquizRelationTable, $condition);
        }
        if (!empty($insert)) {
            $data = [];
            foreach ($insert as $detailsId => $position) {
                $data[] = [
                    'filoblu_scratchandquiz_id' => (int) $id,
                    'filoblu_scratchandquiz_details_id' => (int) $detailsId,
                    'position' => (int) $position['position']
                ];
            }
            $adapter->insertMultiple($this->scratchandquizRelationTable, $data);
        }
        if (!empty($update)) {
            foreach ($update as $detailsId => $position) {
                $where = ['filoblu_scratchandquiz_id = ?' => (int) $id, 'filoblu_scratchandquiz_details_id = ?' => (int) $detailsId];
                $bind = ['position' => (int) $position['position']];
                $adapter->update($this->scratchandquizRelationTable, $bind, $where);
            }
        }
        if (!empty($insert) || !empty($delete)) {
//            $imageIds = array_unique(array_merge(array_keys($insert), array_keys($delete)));
            $this->eventManager->dispatch(
                'filoblu_scratchandquiz_change_details', [
                    'scratchandquiz' => $scratchAndQuiz,
                    'details_ids_insert' => $insert,
                    'details_ids_delete' => $delete,
                    'details_ids_update' => $update,
                ]
            );
        }
        if (!empty($insert) || !empty($update) || !empty($delete)) {
            $scratchAndQuiz->setIsChangedDetailsList(true);
            $detailsIds = array_keys($insert + $delete + $update);
            $scratchAndQuiz->setAffectedDetailsIds($detailsIds);
        }

        return $this;
    }

    public function getDetailsPosition(\FiloBlu\ScratchAndQuiz\Model\ScratchAndQuiz $scratchAndQuiz) {
        $select = $this->getConnection()->select()->from(
            $this->scratchandquizRelationTable, ['filoblu_scratchandquiz_details_id', 'position']
        )
            ->where(
                'filoblu_scratchandquiz_id = :filoblu_scratchandquiz_id'
            );
        $bind = ['filoblu_scratchandquiz_id' => (int) $scratchAndQuiz->getId()];

        return $this->getConnection()->fetchPairs($select, $bind);
    }

}
