<?php

namespace FiloBlu\ScratchAndQuiz\Model;

use Magento\Framework\Model\AbstractModel;

class ScratchAndQuiz extends AbstractModel {

    const CACHE_TAG = 'fb_scratchandquiz';

    protected $_cacheTag = self::CACHE_TAG;

    protected $scratchAndQuizCollectionFactory;
    protected $scratchAndQuizDetailsCollectionFactory;
    protected $scratchAndQuizTranslationsFactory;
    protected $scratchAndQuizDetailsTranslationsFactory;
    protected $storeManager;
    private $objectFactory;

    public function __construct(        
        \Magento\Framework\Model\Context $context, 
        \Magento\Framework\Registry $registry,
        \FiloBlu\ScratchAndQuiz\Model\ResourceModel\ScratchAndQuiz\CollectionFactory $scratchandquizCollectionFactory,
        \FiloBlu\ScratchAndQuiz\Model\ResourceModel\ScratchAndQuizDetails\CollectionFactory $scratchAndQuizDetailsCollectionFactory,
        \FiloBlu\ScratchAndQuiz\Model\ScratchAndQuizTranslationsFactory $scratchAndQuizTranslationsFactory,
        \FiloBlu\ScratchAndQuiz\Model\ScratchAndQuizDetailsTranslationsFactory $scratchAndQuizDetailsTranslationsFactory,
        \Magento\Store\Model\StoreManagerInterface $storemanager,
        \Magento\Framework\DataObjectFactory $objectFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->scratchAndQuizCollectionFactory = $scratchandquizCollectionFactory;
        $this->scratchAndQuizDetailsCollectionFactory = $scratchAndQuizDetailsCollectionFactory;
        $this->scratchAndQuizTranslationsFactory = $scratchAndQuizTranslationsFactory;
        $this->scratchAndQuizDetailsTranslationsFactory = $scratchAndQuizDetailsTranslationsFactory;
        $this->storeManager = $storemanager;
        $this->objectFactory = $objectFactory;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct() {
        $this->_init('FiloBlu\ScratchAndQuiz\Model\ResourceModel\ScratchAndQuiz');
    }

    public function getAvailableStatuses() {
        return array(
            0 => "Disabled",
            1 => "Enabled"
        );
    }

    public function getDetailsPosition() {
        if (!$this->getId()) {
            return array();
        }
        $array = $this->getData('details_position');
        if (is_null($array)) {
            $array = $this->getResource()->getDetailsPosition($this);
            $this->setData('details_position', $array);
        }

        return $array;
    }

    public function getScratchAndQuizByCode($code)
    {

        $collection = $this->scratchAndQuizCollectionFactory->create();
        $collection->addFieldToFilter('enabled', ["eq" => 1]);
        $collection->addFieldToFilter('code', ["eq" => $code]);

        $scratchandquiz = $collection->getFirstItem();

        $scratchandquizEntity = $this->objectFactory->create();

        if ($scratchandquiz) {
            $scratchandquizTranslations = $this->scratchAndQuizTranslationsFactory->create()->loadByParentAndStoreId($scratchandquiz->getId(), $this->storeManager->getStore()->getId());
            if ($scratchandquizTranslations->getId()) {
                $translations = unserialize($scratchandquizTranslations->getSerializedValues());
                $scratchandquizEntity->setData(array_merge($scratchandquiz->getData(), $translations));
            } else {
                $scratchandquizEntity->setData($scratchandquiz->getData());
            }
            
            return $scratchandquizEntity;
        } else {
            return null;
        }
    }

    public function getScratchAndQuizDetails($id) {
        
        $collection = $this->scratchAndQuizDetailsCollectionFactory->create();
        $collection->addFieldToFilter('enabled', ["eq" => 1]);            
    
        $collection->getSelect()->joinInner(
            array('related' => $collection->getTable('filoblu_scratchandquiz_relations')),
            'related.filoblu_scratchandquiz_details_id=main_table.id AND related.filoblu_scratchandquiz_id='.$id,
            array('filoblu_scratchandquiz_id')
        );

        $collection->setOrder('position','ASC');        

        //$scratchandquizdetailsEntity = $this->objectFactory->create();

        foreach($collection as $item) {
            $scratchandquizdetailsTranslations = $this->scratchAndQuizDetailsTranslationsFactory->create()->loadByParentAndStoreId($item->getId(), $this->storeManager->getStore()->getId());
            if ($scratchandquizdetailsTranslations->getId()) {
                $translations = unserialize($scratchandquizdetailsTranslations->getSerializedValues());
                $item->setData(array_merge($item->getData(), $translations));
            } 

            //$scratchandquizdetailsEntity->setData($item->getData());
        }
        
        return $collection;
    }

    public function save()
    {
        $this->_cacheTag = self::CACHE_TAG . '_' . $this->getId();
        return parent::save();
    }

    public function afterLoad()
    {
        $this->_cacheTag = self::CACHE_TAG . '_' . $this->getId();
        return parent::_afterLoad();
    }

    public function beforeDelete()
    {
        $this->_cacheTag = self::CACHE_TAG . '_' . $this->getId();
        return parent::beforeDelete();
    }
}
