<?php

namespace FiloBlu\ScratchAndQuiz\Model;

class ScratchAndQuizDetails extends \Magento\Framework\Model\AbstractModel {

    const CACHE_TAG = 'fb_scratchandquiz';

    protected $_cacheTag = self::CACHE_TAG;

    protected $scratchAndQuizDetailsCollectionFactory;

    public function __construct(        
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\ScratchAndQuiz\Model\ResourceModel\ScratchAndQuizDetails\CollectionFactory $scratchAndQuizDetailsCollectionFactory,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {    
        $this->scratchAndQuizDetailsCollectionFactory = $scratchAndQuizDetailsCollectionFactory;    
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct() {
        $this->_init('FiloBlu\ScratchAndQuiz\Model\ResourceModel\ScratchAndQuizDetails');
    }

    public function getAvailableStatuses() {
        return array(
            0 => "Disabled",
            1 => "Enabled"
        );
    }        

    public function getScratchAndQuizIdRelations($id) {
        
        $collection = $this->scratchAndQuizDetailsCollectionFactory->create();
        $collection->addFieldToFilter('id', ["eq" => $id]);            
    
        $collection->getSelect()->joinInner(
            array('related' => $collection->getTable('filoblu_scratchandquiz_relations')),
            'related.filoblu_scratchandquiz_details_id=main_table.id',
            array('filoblu_scratchandquiz_id')
        );

        //$collection->addFieldToSelect('related.filoblu_scratchandquiz_id');            

        $this->_cacheTag = [];        

        foreach($collection as $item) {            
            $this->_cacheTag[] = self::CACHE_TAG . '_' . $item->getFilobluScratchandquizId();
        }

        return $this->_cacheTag;
    }

    public function save()
    {

        $this->getScratchAndQuizIdRelations($this->getId());
        return parent::save();
    }

    public function afterLoad()
    {
        $this->getScratchAndQuizIdRelations($this->getId());
        return parent::_afterLoad();
    }

    public function beforeDelete()
    {
        $this->getScratchAndQuizIdRelations($this->getId());
        return parent::beforeDelete();
    }

}
