define([
    "ko",
    "jquery",
    "jquery/ui",
    "prototype",
    "filoblu_scratchandquiz_lib",
    'FiloBlu_ScratchAndQuiz/js/model/current-date',
    'matchMedia'
], function(ko, $, ui, prototype, wScratchPad, modelDate, mediaCheck) {
    "use strict";

    $.widget('filoblu.scratchandquiz', {
        options: {
            identifier: null,
            parentCode: null,
            container: null,
            promocode: null,
            shortText: null,
            longText: null,
            bgImage: null,
            overlayImage: null,
            cursor: null,
            storage_key: 'scratchedquiz',
            cursorSize: 80,
            percentage: 50,
            expirationDay: null,
            expirationHour: null,
            ctaLink: null,
            right_detail: null,
            winHtml: null,
            looseHtml: null,
        },

        _create: function() {

            var self = this;

            ko.computed(function() {
                self.checkDate(modelDate());
            }, this);

        },

        checkDate: function(date) {

            if (!date) {
                return;
            }
            var self = this;
            return self.initScratch(true);

        },

        handleAnswer:function(answer, self){

            self.showScratched();

            $(".scratchandquiz-banner-quiz .scratchpad-box-container ").hide();
            $(self.element).parent(".scratchpad-box-container").show();

            $(self.element).parent(".scratchpad-box-container").addClass(answer ? "win" : "loose" )

            $(answer ? self.options.winHtml : self.options.looseHtml ).fadeIn();

            self.appendPromoCode(self)
        },

        initScratch: function(init) {
            var self = this,
                show = false,
                promoTextGA;

            if (init) {

                // show placeholder
                $('.placeholder', self.element).fadeIn();
                $('.scratch-out-text', $(self.element).parent()).fadeIn();

                // check if isset cookie & init scratchPad
                if (!$.cookieStorage.get(self.options.storage_key) || $.cookieStorage.get(self.options.storage_key) != self.options.identifier) {

                    $(self.element).parent().addClass('active');

                    $(self.element).wScratchPad({
                        // the size of the eraser
                        size: self.options.cursorSize,
                        // the randomized scratch image   
                        bg: self.options.bgImage,
                        // give real-time updates
                        realtime: true,
                        // The overlay image
                        fg: self.options.overlayImage,
                        // The cursor (coin) image
                        'cursor': 'url(' + self.options.cursor + ') 5 5, default',

                        scratchMove: function(e, percent) {

                            // Show the plain-text promo code and call-to-action when the scratch area is 50% scratched
                            if ((percent > self.options.percentage) && (self.options.shortText != '') && !show) {

                                self.handleAnswer(self.options.right_detail === self.options.identifier, self)

                                if (self.options.shortText) {
                                    promoTextGA = $('<div>').html(self.options.shortText).text();
                                } else {
                                    promoTextGA = self.options.identifier;
                                }

                                if (self.options.longText) {
                                    $('.scratch-promo-description', self.options.container).addClass('show').append(self.options.longText);

                                    mediaCheck({
                                        media: '(max-width: 991px)',
                                        entry: $.proxy(function() {

                                            $('html,body').animate({
                                                'scrollTop': ($('.scratch-promo-description', self.options.container).offset().top - $('.page-header').outerHeight())
                                            });

                                        }, this),
                                        exit: $.proxy(function() {
                                            // nothing
                                        }, this)
                                    });
                                }

                                if (self.options.ctaLink) {
                                    $(self.element).wrap("<a href='" + self.options.ctaLink + "'></a>");
                                }

                                if (window.ga) {
                                    window.ga('send', {
                                        hitType: 'event',
                                        eventCategory: 'ScratchAndQuiz',
                                        eventAction: 'click',
                                        eventLabel: self.options.parentCode + ' - ' + promoTextGA
                                    });
                                }

                                window.dataLayer = window.dataLayer || [];
                                dataLayer.push({
                                    'event': 'scratchandquiz',
                                    'scratchandquiz': {
                                        'category': 'ScratchAndQuiz',
                                        'action': 'click',
                                        'label': self.options.parentCode + ' - ' + promoTextGA
                                    }
                                });

                                show = true;

                                $.cookieStorage.set(self.options.storage_key, self.options.identifier);
                            }
                        }

                    });

                } else {

                    this.handleAnswer($.cookieStorage.get(self.options.storage_key) === self.options.right_detail, self)

                    // if daily scratchpad has been scratched show scratched image
                    $(self.element).parent().addClass('scratched');
                    $('.placeholder', self.element).remove();
                    $('.placeholder-scratched', self.element).fadeIn();
                    $('.scratch-out-text', $(self.element).parent()).fadeIn();
                    if (self.options.longText) {
                        $('.scratch-promo-description', self.options.container).addClass('show').append(self.options.longText);
                    }

                    $(self.element).parent().addClass('active');

                    if (typeof window.scratchSlider != 'undefined') {
                        var index = $(self.element).parent().parent().attr('data-index');
                        scratchSlider.slideTo(index);
                    }
                }

            }

        },

        showPlaceholder: function() {
            var self = this;

            // future scratchpad
            $(self.element).parent().addClass('to-scratch');
            $('.placeholder', self.element).fadeIn();
            $('.scratch-out-text', $(self.element).parent()).fadeIn();
        },

        showScratched: function() {
            var self = this;

            // past scratchpad
            $(self.element).parent().addClass('scratched');
            $('.placeholder-scratched', self.element).fadeIn();
            $('.scratch-promo', self.element).append(self.options.shortText);
            $('.scratch-out-text', $(self.element).parent()).fadeIn();
        },

        appendPromoCode: function (self){

            $(".promocode-area").fadeIn();
            $(".promocode-area").html(self.options.promocode);
            $(".promo-section img").click( function () {

                var element = $(this);

                $(this).css("opacity", 0)
                setTimeout( function (){
                    $(element).css("opacity", 1)
                    }, 300)
                var $temp = $("<input>");
                $("body").append($temp);
                $temp.val(self.options.promocode).select();
                document.execCommand("copy");
                $temp.remove();
            });

        }

    });

    return $.filoblu.scratchandquiz;
});